/**********
 PCF8563.c
**********/


//***********
//* Includes
//***********
#include "PCF8563.h"

tRTC RTC;

/**
* \brief
* Schreibt die Zeit und das Datum in den RTC
* \note
* Die Daten werden im Burst Modus uebertragen, das heisst, dass nur die erste Adresse geschrieben wird,
* danach werdne alle Daten nacheinander gesendet, da der RTC eine Adress Auto-Increment Funktion besitzt.
* \param
* Sec		Sekunden (0-59)
* \param
* Min		Minuten (0-59)
* \param
* Hour		Stunden (0-23)
* \param
* Day		Tag (1-31)
* \param
* Weekday	Wochentag (0-6)
* \param
* Month		Monat (1-12)
* \param
* Year		Jahr (0-99)
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_SetTime
unsigned char  PCF8563_SetTime(	unsigned char Sec,
								unsigned char Min,
								unsigned char Hour,
								unsigned char Day,
								unsigned char Weekday,
								unsigned char Month,
								unsigned char Year )
{
	unsigned char State;
	
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(ADDR_SECONDS);
	i2c_write(BinaryToBCD(Sec));
	i2c_write(BinaryToBCD(Min));
	i2c_write(BinaryToBCD(Hour));
	i2c_write(BinaryToBCD(Day));
	i2c_write(BinaryToBCD(Weekday));
	i2c_write(BinaryToBCD(Month));
	State = i2c_write(BinaryToBCD(Year));
	i2c_stop();
	
	return State;
}
#endif


/**
* \brief
* Liest die Zeit und das Datum asu dem RTC
* \note
* Die Daten werden im Burst Modus uebertragen, das heisst, dass nur die erste Adresse geschrieben wird,
* danach werdne alle Daten nacheinander gesendet, da der RTC eine Adress Auto-Increment Funktion besitzt.
*/
#ifndef DONOTUSE_PCF8563_GetTime
void PCF8563_GetTime( void )
{
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(ADDR_SECONDS);
    i2c_rep_start(PCF8563_ADDR_Read);
    RTC.Sec = BCDToBinary(i2c_readAck() & MASK_SECONDS);
    RTC.Min  = BCDToBinary(i2c_readAck() & MASK_MINUTES); 
	RTC.Hour  = BCDToBinary(i2c_readAck() & MASK_HOURS); 
	RTC.Day = BCDToBinary(i2c_readAck() & MASK_DAY); 
	RTC.Weekday = BCDToBinary(i2c_readAck() & MASK_WEEKDAY); 
	RTC.Month  = BCDToBinary(i2c_readAck() & MASK_MONTH);                
   	RTC.Year = BCDToBinary(i2c_readNak() & MASK_YEAR);                 
    i2c_stop();
}
#endif


/**
* \brief
* Stellt die Frequenz des Clock Ausgangs ein <br>
* Der Ausgang wird aktiviert
* \param
* ClockSetting		0x00=32768 Hz, 0x01=1024 Hz, 0x02=32Hz, 0x03=1Hz
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_SetClockOut
unsigned char PCF8563_SetClockOut( unsigned char ClockSetting )
{
	unsigned char State;
	
	ClockSetting |= MASK_CLOCK_ENABLE;
	
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(ADDR_CLOCK);
	State = i2c_write(ClockSetting);
	i2c_stop();
	
	return State;
}
#endif


/**
* \brief
* Deaktiviert den ClockAusgang
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_DisableClockOut
unsigned char PCF8563_DisableClockOut( void )
{
	unsigned char State;
	
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(ADDR_CLOCK);
	State = i2c_write(MASK_CLOCK_DISABLE);
	i2c_stop();
	
	return State;
}
#endif


/**
* \brief
* Schreibt die Alarmzeit in den RTC <br>
* Es koennen Minuten, Stunden, Tag, und Wochentag programmiert werden. Wenn alle Aktivierten <br>
* Werte mit der aktuellen Uhrzeit uebereinstimmen, dann wir ein Alarm ausgeloest.
* \note
* Der Alarm selbel muss noch mit der Funktion PCF8563_EnableAlarmInterrupt aktiviert werden. <br>
* Der Alarm ist solange aktiv, bis der Alarm mit PCF8563_ClearAlarmFlag zurueckgesetzt wird.
* \param
* Min		Minuten (0-59)
* \param
* Hour		Stunden (0-23)
* \param
* Day		Tag (1-31)
* \param
* Weekday	Wochentag (0-6)
* \param
* IsEnMin	ALARM_ENABLE (0xff) oder ALARM_DISABLE (0x7f)
* \param
* IsEnHour	ALARM_ENABLE (0xff) oder ALARM_DISABLE (0x7f)
* \param
* IsEnDay	ALARM_ENABLE (0xff) oder ALARM_DISABLE (0x7f)
* \param
* IsEnWeekday	ALARM_ENABLE (0xff) oder ALARM_DISABLE (0x7f)
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_SetAlarmTime
unsigned char  PCF8563_SetAlarmTime(	unsigned char Min,
										unsigned char Hour,
										unsigned char Day,
										unsigned char Weekday,
										unsigned char IsEnMin,
										unsigned char IsEnHour,
										unsigned char IsEnDay,
										unsigned char IsEnWeekday )
{
	unsigned char State;
	
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(ADDR_ALARM_MINUTE);
	i2c_write(BinaryToBCD(Min) | IsEnMin);
	i2c_write(BinaryToBCD(Hour) | IsEnHour);
	i2c_write(BinaryToBCD(Day) | IsEnDay);
	State = i2c_write(BinaryToBCD(Weekday) | IsEnWeekday);
	i2c_stop();
	
	return State;
}
#endif


/**
* \brief
* Aktiviert den Interrupt,  der bei einem Alarm ausgeloest wird
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_EnableAlarmInterrupt
unsigned char PCF8563_EnableAlarmInterrupt( void )
{
	unsigned char ControlRegister;
	unsigned char State;
	
	ControlRegister = PCF8563_ReadRegister(ADDR_CONTROL2);
	
	// AIE Bit setzen
	ControlRegister |= ALARM_INTERRUPT;
	
	State = PCF8563_WriteRegister(ADDR_CONTROL2, ControlRegister);
	
	return State;
}
#endif


/**
* \brief
* Deaktiviert den Interrupt, wenn der Alarm ausgeloest wird
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_DisableAlarmInterrupt
unsigned char PCF8563_DisableAlarmInterrupt( void )
{
	unsigned char ControlRegister;
	unsigned char State;
	
	ControlRegister = PCF8563_ReadRegister(ADDR_CONTROL2);
	
	// AIE Bit loeschen
	ControlRegister &= ~ALARM_INTERRUPT;
	
	State = PCF8563_WriteRegister(ADDR_CONTROL2, ControlRegister);
	
	return State;
}
#endif


/**
* \brief
* Schreibt ein einzelnes Register
* \param
* Addr Adresse des Registers
* \param
* Data Datenbyte welches in das Register geschrieben werden soll
* \return
* 0x00 Wenn der Vorgang erfolgreich war
*/
#ifndef DONOTUSE_PCF8563_WriteRegister
unsigned char PCF8563_WriteRegister( unsigned char Addr, unsigned char Data)
{
	unsigned char State;
	
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(Addr);
	State = i2c_write(Data);
	i2c_stop();
	
	return State;
}
#endif


/**
* \brief
* Liest ein einzelnes Register
* \param
* Addr Adresse des Registers
* \return
* Datenbyte des Registers
*/
#ifndef DONOTUSE_PCF8563_ReadRegister
unsigned char PCF8563_ReadRegister( unsigned char Addr)
{
	unsigned char Data;
	
	i2c_start(PCF8563_ADDR_Write);
	i2c_write(Addr);
	i2c_rep_start(PCF8563_ADDR_Read);              
	Data = (i2c_readNak());                 
	i2c_stop();
	
	return Data;
}
#endif


/**
* \brief
* Wandelt eine BCD-Zahl in eine Binaer-Zahl um
* \param BCDCode <br>
* LowByte: Einer <br>
* HighByte: Zehner
* \return
* Binaerzahl zwischen 0 und 99
*/
#ifndef DONOTUSE_PCF8563_BCDToBinary
unsigned char BCDToBinary(unsigned char BCDCode)
{
	unsigned char BinaryCode;
	unsigned char Einer;
	unsigned char Zehner;

	// Einer (LowByte) ausmaskieren
	Einer = BCDCode & 0x0F;

	// Zehner (HighByte) ausmaskieren
	Zehner = BCDCode & 0xF0;

	// Um 4 nach rechts schieben (/16)
	Zehner /= 16;

	// Binary Code erstellen
	BinaryCode = Zehner;
	BinaryCode *= 10;
	BinaryCode += Einer;

	return BinaryCode;
}
#endif


/**
* \brief
* Wandelt eine Binaer-Zahl in eine BCD-Zahl um
* \param BinaryCode (Binaerzahl zwischen 0 und 99)
* \return
* BCD-Zahl <br>
* LowByte: Einer <br>
* HighByte: Zehner
*/
#ifndef DONOTUSE_PCF8563_BinaryToBCD
unsigned char BinaryToBCD(unsigned char BinaryCode)
{
	unsigned char Einer;
	unsigned char Zehner;
	unsigned char BCDCode;
	unsigned char Zahl;

	Zehner = 0;
	Zahl = BinaryCode;

	// Zehner abzaehlen
	while(Zahl >= 10)
	{
		Zehner++;
		Zahl -= 10;
	}

	// Einer auslesen
	Einer = Zahl;

	BCDCode = Zehner;

	// Um 4 nach links schieben
	BCDCode *= 16;

	// Einer dazuzaehlen
	BCDCode += Einer;

	return BCDCode;
}
#endif
