/**
* \file
* PCF8563.h
* \brief
* Library um den PCF8563 RTC von NXP ueber IIC anzusteuern <br>
* Zur Hardwaresteuerung wird die Library von Peter Fleury <pfleury@gmx.ch>  http://jump.to/fleury verwendet
* \version 
* v. 1.1 Jede Funktion ist einzeln abschaltbar um Speicherplatz zu sparen <br>
* v. 1.0 Erste Version
* \date
* Create: 23.09.2008 <br>
* Modify: 01.01.2009
* \author
* Philipp Kaelin 
*/

#ifndef _PCF8563_H
	#define _PCF8563_H

	//***********
	//* Includes
	//***********
	#include <avr/io.h>
	#include "i2cmaster.h"
	#include "global.h"

	//************
	//* Variabeln
	//************

	/** Zeitangaben */
	typedef struct 
	{
		/** Sekunden 0-59 */
		unsigned char	Sec;
		/** Minuten in 0-59 */
		unsigned char	Min;
		/** Stunden in 0-23 */
		unsigned char	Hour;
		/** Wochentag 0=Sonntag, 6=Samstag */
		unsigned char	Weekday;
		/** Tag 1-31 */
		unsigned char	Day;
		/** Monat 1-12 */
		unsigned char	Month;
		/** Jahr 0-99 , 0=2000 */
		unsigned char	Year;
	} tRTC;	

	extern tRTC RTC;

	//**********
	//* Defines
	//**********
	
	// IIC Hardware Adressen des PCF8563
	// ----------------------------------------------
	// Schreibadresse
	#define PCF8563_ADDR_Write				0xa2
	
	// Leseadresse
	#define PCF8563_ADDR_Read				0xa3
	
	// RAM Adressen
	// ------------------
	// Status und Control Register
	#define ADDR_CONTROL1					0x00
	#define ADDR_CONTROL2					0x01
	
	// Aktuelle Zeit
	#define ADDR_SECONDS					0x02
	#define ADDR_MINUTES					0x03
	#define ADDR_HOURS						0x04
	#define ADDR_DAY						0x05
	#define ADDR_WEEKDAY					0x06
	#define ADDR_MONTH						0x07
	#define ADDR_YEAR						0x08
	
	// Alarmzeit
	#define ADDR_ALARM_MINUTE				0x09
	#define ADDR_ALARM_HOURS				0x0a
	#define ADDR_ALARM_DAY					0x0b
	#define ADDR_ALARM_WEEKDAY				0x0c
	
	// Clock Out
	#define ADDR_CLOCK						0x0d
	
	// Timer
	#define ADDR_TIMER_CONTROL				0x0e
	#define ADDR_TIMER						0x0f
	
	
	// RAM Inhalt Masken
	// -----------------------
	// Aktuelle Zeit
	#define MASK_SECONDS					0x7f
	#define MASK_MINUTES					0x7f
	#define MASK_HOURS						0x3f
	#define MASK_DAY						0x3f
	#define MASK_WEEKDAY					0x07
	#define MASK_MONTH						0x1f
	#define MASK_YEAR						0xff
	
	// Clock Out
	#define MASK_CLOCK_ENABLE				0x83
	#define MASK_CLOCK_DISABLE				0x00

	// Timer
	#define MASK_TIMER_CONTROL				0x03
	#define MASK_TIMER						0xff
	
	// Allgemeine Masken
	// ----------------------
	// Alarmzeit
	#define ALARM_ENABLE					0xff
	#define ALARM_DISABLE					0x7f
	#define ALARM_INTERRUPT					0x02
	
	//******************
	//* Speicher sparen
	//******************
	//#define DONOTUSE_PCF8563_SetTime
	//#define DONOTUSE_PCF8563_GetTime
	//#define DONOTUSE_PCF8563_SetClockOut
	#define DONOTUSE_PCF8563_DisableClockOut
	#define DONOTUSE_PCF8563_SetAlarmTime
	#define DONOTUSE_PCF8563_EnableAlarmInterrupt
	#define DONOTUSE_PCF8563_DisableAlarmInterrupt
	//#define DONOTUSE_PCF8563_WriteRegister
	//#define DONOTUSE_PCF8563_ReadRegister
	//#define DONOTUSE_PCF8563_BCDToBinary
	//#define DONOTUSE_PCF8563_BinaryToBCD
	
	//*************
	//* Funktionen
	//*************
	unsigned char  PCF8563_SetTime(	unsigned char Sec,
									unsigned char Min,
									unsigned char Hour,
									unsigned char Day,
									unsigned char Weekday,
									unsigned char Month,
									unsigned char Year );
	
	void PCF8563_GetTime( void );

	unsigned char PCF8563_SetClockOut( unsigned char ClockSetting );
	
	unsigned char PCF8563_DisableClockOut( void );
	
	unsigned char  PCF8563_SetAlarmTime(unsigned char Min,
										unsigned char Hour,
										unsigned char Day,
										unsigned char Weekday,
										unsigned char IsEnMin,
										unsigned char IsEnHour,
										unsigned char IsEnDay,
										unsigned char IsEnWeekday );

	unsigned char PCF8563_EnableAlarmInterrupt( void );
	
	unsigned char PCF8563_DisableAlarmInterrupt( void );

	unsigned char PCF8563_WriteRegister( unsigned char Addr, unsigned char Data);
	
	unsigned char PCF8563_ReadRegister( unsigned char Addr);

	unsigned char BCDToBinary(unsigned char BCDCode);

	unsigned char BinaryToBCD(unsigned char BinaryCode);
	
#endif
