/*******
dogm.c
*******/

//***********
//* Includes
//***********
#include "dogm.h"

/**
 * \brief
 * Loesch das LCD
*/
void Disp_Clear ( void )
{
	// Display loeschen
	Disp_Command(0x01);
	
	// Auf Display warten
	_delay_ms(25);
	
	// Entry Mode set - Display wartet auf Eingabe
	Disp_Command(0x06);
	
	// Auf Display warten
	_delay_ms(15);
}


/**
 * \brief
 * Loesch eine Zeile auf dem LCD
 * \param
 * ucLine (Zeilennummer 0 bis 255)
*/
void Disp_Clear_Line ( unsigned char ucLine )
{
	unsigned char i;
	
	// Adresse setzen
	Disp_SetAddr(ucLine, 0);
	
	// Leerzeichen schreiben
	for (i=0; i<16; i++)
	{
		Disp_Char(' ');
	}
}


/**
 * \brief
 * Schreibt ein Zeichen auf das LCD
 * \param
 * cData (Zeichenl)
*/
void Disp_Char ( char cData )
{
	// Data Mode vorbeireiten; setze RS high
	PORT_RS = (1 << PINNR_RS);

	// Starte Uebertragung
	SPI_SendByte(cData);
	
	// warte auf Display
	_delay_us(50);
}


/**
 * \brief
 * Schreibt ein String auf das LCD
 * \param
 * *cData (Pointer auf den String)
*/
void Disp_String ( char *cData )
{
	while (*cData)
	{
		Disp_Char(*cData);
		cData++;
	}
}


/**
 * \brief
 * Schreibt eine Zahl auf das LCD
 * \param
 * iData Zahl (0 bis 32768)
*/
void Disp_Integer ( int iData )
{
	char buffer[6];

	utoa(iData, buffer, 10);
	Disp_String(buffer);
}


/**
 * \brief
 * Schreibt eine Zahl mit 2 Stellen auf das LCD
 * \param
 * iData Zahl (0 bis 32768)
*/
void Disp_Integer00 ( int iData )
{
	if (iData < 10)
	{
		Disp_Char('0');
	}
	Disp_Integer(iData);
}


/**
 * \brief
 * Sendet einen Befehl an das LCD
 * \param
 * ucData (Befehl)
*/
void Disp_Command ( unsigned char ucData )
{
	// Command Mode vorbeireiten; setze RS low
	PORT_RS &= ~(1 << PINNR_RS);

	// Starte Uebertragung
	SPI_SendByte(ucData);

	// warte auf Display
	_delay_us(50); 
}


/**
 * \brief
 * Sendet ein Datenbyte an das LCD
 * \param
 * ucData (Datenbyte)
*/
void Disp_Byte ( unsigned char ucData )
{
	// Data Mode vorbeireiten; setze RS high
	PORT_RS = (1 << PINNR_RS);

	// Starte Uebertragung
	SPI_SendByte(ucData);

	// warte auf Display
	_delay_us(50);
}


/**
 * \brief
 * Setzt die Position auf dem LCD
 * \param
 * ucLine Zeile (0 bis 255)
 * \param
 * ucRow Spalte (0 bis 255)
*/
void Disp_SetAddr ( unsigned char ucLine, unsigned char ucRow )
{
	unsigned char Addr;
	
	// Adresse errechnen
	Addr = ucRow;
	while (ucLine)
	{
		Addr += 0x40;
		ucLine--;
	}
	
	// Vorderstes Bit immer auf 1
	Addr |= 0x80;
	
	// Adresse senden
	Disp_Command(Addr);
}


/**
 * \brief
 * Initialisiert das LCD
 * \param Contrast 0x73 bis 0c7F
*/
void Disp_Init ( unsigned char Contrast )
{
	// Warte auf Display
	_delay_ms(80);

	// Function Set, 8Bit, 2 Lines, No Double Height, Extendent Instruction Table 
	Disp_Command(0x39);

	// Bias Set, 1/4
	Disp_Command(0x1c);

	// Power Controll, Booster On, Contrast C5, set C4
	Disp_Command(0x55);

	// Follower Controll, Follower On, 
	Disp_Command(0x6d);

	// Contrast, 0x73 to 0x7F
	Disp_Command(Contrast);

	// Display on, no Cursor, no Blink
	Disp_Command(0x0c);

	// Clear LCD
	Disp_Command(0x01);

	// Entry Mode set
	Disp_Command(0x06);
}


/**
 * \brief
 * Initialisiert die SPI-Schnittstelle
*/
void Disp_InitHW ( void )
{
	// Setze SI als Ausgang und low
	DDR_SI |= (1 << PINNR_SI);
	PORT_SI &= ~(1 << PINNR_SI);

	// Setze CLK als Ausgang und low
	DDR_CLK |= (1 << PINNR_CLK);
	PORT_CLK &= ~(1 << PINNR_CLK);

	// Setze CSB als Ausgang und high
	DDR_CSB |= (1 << PINNR_CSB);
	PORT_CSB |= (1 << PINNR_CSB);

	// Setze RS-Pin als Ausgang und low
	DDR_RS |= (1 << PINNR_RS);
	PORT_RS &= ~(1 << PINNR_RS);

	// Backlight-Pin als Ausgang und low
	DDR_BACKLIGHT |= (1 << PINNR_BACKLIGHT);
	PORT_BACKLIGHT &= ~(1 << PINNR_BACKLIGHT);
}


/**
 * \brief
 * Sendet ein Byte ueber die SPI-Schnittstelle
*/
void SPI_SendByte( unsigned char  ucData )
{
	unsigned char i;

	// Select Chip
  	PORT_CSB &= ~(1 << PINNR_CSB);

  	for( i=8; i>0; i-- )
	{
		// CLK low
    	PORT_CLK &= ~(1 << PINNR_CLK);

		// SI low
    	PORT_SI &= ~(1 << PINNR_SI);

		// Ckeck if Bit7 is high
    	if( ucData & 0x80 )
		{
			// Set SI high
      		PORT_SI |= (1 << PINNR_SI);
		}

		// Shift d 1 bit to left
    	ucData <<= 1;

		// CLK high
    	PORT_CLK |= (1 << PINNR_CLK);
  	}

	// UnSelect Chip
 	PORT_CSB |= (1 << PINNR_CSB);
}

/**
 * \brief
 * Stellt den Kontrast in abhaengigkeit der Spannung ein
*/
void Disp_AdjustContrast ( void )
{
	TaskScheduler_RemoveTask(Disp_AdjustContrast);
	TaskScheduler_AddTask(Disp_AdjustContrast, 3600);
	
	unsigned char Contrast;
	unsigned int UCount;

	Read_Voltages();

	UCount = 3200;
	Contrast = 120;
	while (Contrast < 127)
	{
		if (Voltage_Batt < UCount)
		{
			UCount -= 50;
			Contrast++;	
		}
		else
		{
			break;
		}
	}

	Disp_Init(Contrast);
}


/**
 * \brief
 * Aktiviert einen blinkenden Cursor
*/
void Disp_Cursor_On ( void )
{
	Disp_Command(0x0f);
}


/**
 * \brief
 * Deaktiviert den Cursor
*/
void Disp_Cursor_Off ( void )
{
	Disp_Command(0x0c);
}


/**
 * \brief
 * Schaltet die Hintergrundbeleuchtung ein
*/
void SetBacklightOn ( void )
{
	PORT_BACKLIGHT |= (1 << PINNR_BACKLIGHT);
}


/**
 * \brief
 * Schaltet die Hintergrundbeleuchtung aus
*/
void SetBacklightOff ( void )
{
	PORT_BACKLIGHT &= ~(1 << PINNR_BACKLIGHT);
}

