/**
* \mainpage
* Wake-Up-Light
*
* <center> Wake-Up-Light </center> <br> <br>
*
* \author
* Philipp Kaelin
*
* \date
* Create: 15.10.2008 <br>
* Modify: 02.05.2009
*
* \version
* v. 1.0
*
* \note
* Processor: ATmega88 <br>
* Flash Size: 8'192 Byte <br>
* SRAM Size: 1024 Byte <br>
* Oscillator: 8 MHz internal Oscillator with divider 8
**/

//***********
//* Includes
//***********
#include <avr/io.h>
#include <avr/interrupt.h>
#include "global.h"
#include "init.h"
#include "dogm.h"
#include "i2cmaster.h"
#include "PCF8563.h"
#include "application.h"
#include "task_handler.h"
#include "scheduler.h"
#include "ColorLight.h"
#include "debug.h"


//**********
//* Defines
//**********


//*************************
//* Funktionsdeklarationen
//*************************



//************
//* Variabeln
//************
unsigned char TaskHandlerState;
unsigned char BuzzerCount;


/**
 * \brief
 * Hauptprogramm
*/
int main ( void )
{
	//******************
	//* Initialisierung
	//******************
	
	// Watchdog deaktivieren
	cli();
	wdt_reset();
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0x00;

	// LCD
	Disp_InitHW();
	Disp_Init(STDCONTRAST);
	
	// RTC
	i2c_init();
	Init_RTCPins();

	// AD Eingaenge
	Init_ADPins();

	// Tasten
	Init_Tasten();

	// Init UART
	#ifdef DEBUG_ENABLED
	UARTInit();
	#endif

	// Stromsparmodis
	#ifndef DEBUG_ENABLED
	PowerSave();
	#endif

	// Interrupt
	Init_PCI();
	sei();

	// Piepser
	Init_Buzzer();
	BuzzerOff();
	BUZZERPORT &= ~(1 << BUZZERPINNR);

	// Schedulerinit
	TaskScheduler_Init();

	// Kontrast einstellen und Spannung messen
	Disp_AdjustContrast();

	//****************
	//* Hauptprogramm
	//****************
	PCF8563_GetTime();
	PCF8563_SetClockOut(0x03);
	ReadAlarmState();

	// Standardanzeige
	ApplicationBusy = TRUE;
	TaskHandler_AddTask(Menue);

	while(1)
	{
		// Task abarbeiten
		TaskHandlerState = TaskHandler_HandleEvents();

		if (TaskHandlerState == TASK_QUEUE_EMPTY)
		{
			if (Alarm.On == TRUE)
			{
				TaskHandler_AddTask(PWMLight);
			}
			else
			{
				// nur schlafen wenn das Licht nicht eingeschaltet ist 
				EnterSleep();
			}
		}
	}
}


/**
 * \brief
 * PinChangeInterrupt 15..8
*/
ISR (PCINT1_vect)
{
	TaskHandler_AddTaskIfNotExist(TastenEinlesen);

	if (ApplicationBusy == FALSE)
	{
		TaskHandler_AddTask(Menue);
	}
}


/**
 * \brief
 * PinChangeInterrupt 23..16
*/
ISR (PCINT2_vect)
{
	// RTC CLK (nur CLK und kein anderer Pin)
	if (((~PIND) & 0x17) == 0x04) 
	{
		Inc_Time();		
		TaskHandler_AddTask(TaskScheduler_TimerTick);
	}
	else
	{
		PCF8563_GetTime();
		
		if (RTC.Sec != Zeit.LastSec)
		{
			// Sekunde hat geaendert

			Zeit.LastSec = RTC.Sec;
			Zeit.SecChanged = TRUE;
			TaskHandler_AddTask(TaskScheduler_TimerTick);

			if (RTC.Min != Zeit.LastMin)
			{
				// Minute hat geaendert

				Zeit.LastMin = RTC.Min;
				Zeit.MinChanged = TRUE;
				TaskHandler_AddTask(CheckAlarm);
			}
		}
	}

	// ueberpruefen, ob ein Taster aktiv ist
	if (~PIND & 0x11)
	{
		TaskHandler_AddTaskIfNotExist(TastenEinlesen);
	}

	if (ApplicationBusy == FALSE)
	{
		TaskHandler_AddTask(Menue);
	}
}


/**
 * \brief
 * Timer0 Overflow Interrupt
*/
ISR (TIMER0_OVF_vect)
{
	ColorCnt++;
}


/**
 * \brief
 * Timer2 Overflow Interrupt 65.5ms
*/
ISR (TIMER2_OVF_vect)
{
	IncChar(&BuzzerCount, 16, 0);

	if ((BuzzerCount < 2) || ((BuzzerCount > 4) && (BuzzerCount < 7)))
	{
		BUZZERPORT |= (1 << BUZZERPINNR);
	}
	else
	{
		BUZZERPORT &= ~(1 << BUZZERPINNR);
	}
}
