/*************
  scheduler.c
 *************/


//***********
//* Includes
//***********
#include "scheduler.h"


//************
//* Variables
//************
volatile struct {
	/** Ringpuffer in wwelchem die Tasks abgespeichert werden */
	pFunction pTask[TASKSCHEDULER_SIZE];
	/** Verbleibende Zeit (SchedulerTicks) bis zur ausfuehrung */
	unsigned int uiRemainTime[TASKSCHEDULER_SIZE];
	/** TRUE wenn ein neuer Task hinzugefuegt wurde und der Array neu geordnet werden muss */
	unsigned char bStackChanged;
} stcTaskScheduler;


//*************
//* Funktionen
//*************
/**
* \brief
* Initialisiert den TaskScheduler
* \note
* Muss aufgrufen werden befor ein Task hinzugefuegt wird
*/
void TaskScheduler_Init ( void )
{
	unsigned char i;
	
	for (i=0; i<TASKSCHEDULER_SIZE; i++)
	{
		stcTaskScheduler.uiRemainTime[i] = 0xffff;
	}
}


/**
* \brief
* TimeTick muss periodisch ausgefuehrt werden
* \note
* Sortiert die Tasks neu falls noetig
*/
void TaskScheduler_TimerTick ( void )
{
	unsigned char i;
	
	// Wenn ein neuer Task hinzugefuegt wurde dann neu sortieren
	if (stcTaskScheduler.bStackChanged == TRUE)
	{
		TaskScheduler_SortTasks();
	}
	
	// Bei allen Tasks die Zeit bis zur ausfuehrung dekrementieren
	for (i=0; i<TASKSCHEDULER_SIZE; i++)
	{
		if (stcTaskScheduler.uiRemainTime[i] != 0xffff)
		{
			stcTaskScheduler.uiRemainTime[i]--;
		}
	}
	
	// Alle Tasks deren Zeit abgelaufen ist ausfuehren
	while (stcTaskScheduler.uiRemainTime[0] == 0)
	{
		// Task ausfuehren
		stcTaskScheduler.pTask[0]();
		
		// Task loeschen und Array neu sortieren
		stcTaskScheduler.uiRemainTime[0] = 0xffff;
		TaskScheduler_SortTasks();
	}
}


/**
* \brief
* Task hinzufuegen
* \param
* TaskToAdd Funktion die ausgefuehrt werden soll
* \param
* uiTimeUntilExecute Zeit bis zur ausfuehrung des Tasks
* \return
* - TASK_ADDED Task hinzugefuegt <br>
* - TASK_QUEUE_FULL der Task konnte nicht hinzugefuegt werden
*/
unsigned char TaskScheduler_AddTask ( pFunction TaskToAdd, unsigned int uiTimeUntilExecute )
{
	unsigned char i = 0;
	
	for (i=0; i<TASKSCHEDULER_SIZE; i++)
	{
		if (stcTaskScheduler.uiRemainTime[i] == 0xffff)
		{
			if (i < TASKSCHEDULER_SIZE)
			{
				stcTaskScheduler.pTask[i] = TaskToAdd;
				stcTaskScheduler.uiRemainTime[i] = uiTimeUntilExecute;
				stcTaskScheduler.bStackChanged = TRUE;
				return TASK_ADDED;
			}
			else
			{
				#ifdef SCHEDULER_RESET_IF_TASK_QUEUE_FULL
					Reset();
				#endif
				return TASK_QUEUE_FULL;
			}
		}
	}
	return TASK_QUEUE_FULL;
}


/**
* \brief
* Task entfernen
* \param
* TaskToRemove Funktion die entfernt werden soll
* \return
* - TASK_REMOVED Task entfern <br>
* - TASK_NOT_FOUND Task nicht gefunden
*/
unsigned char TaskScheduler_RemoveTask ( pFunction TaskToRemove )
{
	unsigned char i = 0;
	
	for (i=0; i<TASKSCHEDULER_SIZE; i++)
	{
		if (stcTaskScheduler.pTask[i] == TaskToRemove)
		{
			stcTaskScheduler.uiRemainTime[i] = 0xffff;
			stcTaskScheduler.bStackChanged = TRUE;
			return TASK_REMOVED;
		}
	}
	return TASK_NOT_FOUND;
}


/**
* \brief
* Sortiert die Tasks
* \note
* Nutzt den BubbleSort Algorithmus
* Falls moeglich sollten die Interrupts waehrend dem sortieren mit
* DISABLE_INTERRUPT_DURING_SORT ausgeschaltet werden, um zu verhinder,
* dass waehrend dem sortieren ein neuer Task hinzugefuegt wird.
*/
void TaskScheduler_SortTasks ( void )
{
	unsigned int i, j;
	unsigned int uiTemp;
	pFunction pTemp;
	
	#ifdef DISABLE_INTERRUPT_DURING_SORT
		cli();
	#endif
	
	for (i=0; i<TASKSCHEDULER_SIZE; i++)
	{
		for (j=0; j<TASKSCHEDULER_SIZE-i-1; ++j)
		{
			if (stcTaskScheduler.uiRemainTime[j] > stcTaskScheduler.uiRemainTime[j + 1])
			{
				uiTemp = stcTaskScheduler.uiRemainTime[j];
				stcTaskScheduler.uiRemainTime[j] = stcTaskScheduler.uiRemainTime[j + 1];
				stcTaskScheduler.uiRemainTime[j + 1] = uiTemp;
				
				pTemp = stcTaskScheduler.pTask[j];
				stcTaskScheduler.pTask[j] = stcTaskScheduler.pTask[j + 1];
				stcTaskScheduler.pTask[j + 1] = pTemp;
			}
		}
	}
	
	#ifdef DISABLE_INTERRUPT_DURING_SORT
		sei();
	#endif
	
	stcTaskScheduler.bStackChanged = FALSE;
}

