/***************
 task_handler.c
****************/


//***********
//* Includes
//***********
#include "task_handler.h"


//************
//* Variabeln
//************
volatile struct
{
	/** Ringpuffer in welchem die Tasks abgespeichert werden */
	pFunction pTask[TASKHANDLER_SIZE];
	/** Adresse im RingBuffer an welcher der juengste Task geschriben wurde.
		Die Adresse muss vor dem schreiben des naechsten Tasks inkrementiert werden */
	unsigned char ucWritePos;
	/** Adresse im RingBuffer auf den Task der gerade oder als letztes abgearbeitet wurde.
		Die Adresse muss vor dem lesen des naechsten Tasks inkrementiert werden */
	unsigned char ucReadPos;
} stcTaskHandler;

// Taskhandler Groesse ueberpruefen
#if ((TASKHANDLER_SIZE > 255) || (TASKHANDLER_SIZE < 1))
		#error TASKHANDLER_SIZE Muss zwischen 1 und 255 liegen
#endif


//*************
//* Funktionen
//*************

/**
* \brief
* Naechster Task in der Queue ausfuehren
* \return
* TASK_QUEUE_PRESENT wenn noch ein oder mehrere Tasks in der Queue sind <br>
* TASK_QUEUE_EMPTY wenn kein Task mehr in der Queue ist
* \note
* Pro Aufruf wird immer nur ein Task abgearbeitet
*/
unsigned char TaskHandler_HandleEvents ( void )
{	
	if (stcTaskHandler.ucWritePos != stcTaskHandler.ucReadPos)
	{
		// Leseadresse inkrementieren
		TaskHandler_IncPos(&stcTaskHandler.ucReadPos);

		// Task ausfuehren
		stcTaskHandler.pTask[stcTaskHandler.ucReadPos]();
		
		// Es sind noch Tasks in der Warteschlange verhanden
		return TASK_QUEUE_PRESENT;
	}
	else
	{
		// Keine Tasks mehr in der Warteschlange
		return TASK_QUEUE_EMPTY;
	}
}


/**
* \brief
* Task hinzufuegen
* \note
* Macht einen Reset wenn die Queue voll ist
* \param
* TaskToAdd Funktion die nizugefuegt werden soll
* \return
* TASK_ADDED wenn Task hinzugefuegt
* TASK_QUEUE_FULL wenn der Task nicht hinzugefuegt wurde
*/
unsigned char TaskHandler_AddTask ( pFunction TaskToAdd )
{
	if ((stcTaskHandler.ucWritePos + 1) != stcTaskHandler.ucReadPos)
	{
		// Schreibadresse inkrementieren
		TaskHandler_IncPos(&stcTaskHandler.ucWritePos);

		// Task hinzufuegen
		stcTaskHandler.pTask[stcTaskHandler.ucWritePos] = TaskToAdd;

		// Task erfolgreich hinzugefuegt
		return TASK_ADDED;
	}
	else
	{
		// TaskHandler Queue Voll (Task nicht hinzugefuegt)
		#if (RESET_IF_TASK_QUEUE_FULL == TRUE)
			Reset();
		#endif
		return TASK_QUEUE_FULL;
	}
}


/**
* \brief
* Task hinzufuegen falls er nicht schon in der Queue ist
* \param
* TaskToAdd Funktion die nizugefuegt werden soll
* \return
* TASK_FOUND wenn der Task schon vorhanden ist
* TASK_ADDED wenn Task hinzugefuegt
* TASK_QUEUE_FULL wenn der Task nicht hinzugefuegt wurde
*/
unsigned char TaskHandler_AddTaskIfNotExist ( pFunction TaskToAdd )
{
	unsigned char ucAdress;
	unsigned char bTaskFound;
	
	ucAdress = stcTaskHandler.ucReadPos;
	bTaskFound = FALSE;
	
	// Task suchen
	while (ucAdress != stcTaskHandler.ucWritePos)
	{
		TaskHandler_IncPos(&ucAdress);
		if (TaskToAdd == stcTaskHandler.pTask[ucAdress])
		{
			bTaskFound = TRUE;
		}
	}
	
	if (bTaskFound == FALSE)
	{
		return TaskHandler_AddTask(TaskToAdd);
	}
	else
	{
		return TASK_FOUND;
	}
}


/**
* \brief
* TaskAdresse inkrementieren
* \param
* *pAddress Pointer auf die Adresse, welche erhoeht werden soll
*/
void TaskHandler_IncPos ( unsigned char *pAddress )
{
	// Adresse erhoehen
	*pAddress = *pAddress + 1;

	// Adresse auf Arrayueberlauf pruefen
	if (*pAddress > (TASKHANDLER_SIZE - 1))
	{
		*pAddress = 0;
	}
}
